@@ -25,7 +25,10 @@ impl YamlBackend {
 		let path = path.as_ref().to_path_buf();
 
 		if path.is_file() {
-			Err(FsError::path_not_directory(path))
+			Err(FsError {
+				source: None,
+				kind: FsErrorType::PathNotDirectory { path },
+			})
 		} else {
 			Ok(Self(path))
 		}
@@ -40,20 +43,14 @@ impl FsBackend for YamlBackend {
 		R: io::Read,
 		T: Entry,
 	{
-		serde_yaml::from_reader(rdr).map_err(|e| FsError {
-			source: Some(Box::new(e)),
-			kind: FsErrorType::Deserialization,
-		})
+		Ok(serde_yaml::from_reader(rdr)?)
 	}
 
 	fn to_bytes<T>(&self, value: &T) -> Result<Vec<u8>, FsError>
 	where
 		T: Entry,
 	{
-		serde_yaml::to_vec(value).map_err(|e| FsError {
-			source: Some(Box::new(e)),
-			kind: FsErrorType::Serialization,
-		})
+		Ok(serde_yaml::to_vec(value)?)
 	}
 
 	fn base_directory(&self) -> PathBuf {
